﻿
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using OfficeOpenXml;
using Newtonsoft.Json;

namespace Bird_tool
{

    public class ExcelDataManager
    {
        public delegate void LogHandler(string message, LogLevel level);
        public delegate void LogShowHandler(string message);
        public event LogHandler OnLog;
        public event LogShowHandler OnLogShow;

        static ExcelDataManager()
        {
            // 设置 EPPlus 许可证
            ExcelPackage.LicenseContext = LicenseContext.NonCommercial;
        }

        private readonly string _excelPath;
        private readonly string _progressPath;
        private readonly List<Dictionary<string, string>> _rows = new List<Dictionary<string, string>>();
        private readonly Dictionary<string, UsageRecord> _usageRecords = new Dictionary<string, UsageRecord>(StringComparer.OrdinalIgnoreCase);
        private int _lastUsedIndex = 0;
        private string[] _headers;

        private readonly AppConfig _config;

        // 验证状态和错误信息
        public bool IsValid { get; private set; }
        public List<string> ValidationErrors { get; } = new List<string>();

        // 主键列名
        private string _primaryKeyColumn = "uuid";

        public ExcelDataManager(AppConfig config, string progressPath = "progress.json")
        {
            _config = config;
            _excelPath = config.excel_path;
            _progressPath = progressPath;
            if (_config.enable_excel_load)
            {
                LoadExcelData();
                ValidateExcelData();
                if (IsValid)
                {
                    LoadProgress();
                }
                else
                {
                    LogValidationErrors();
                }
            }
        }

        private void LoadExcelData()
        {
            if (!File.Exists(_excelPath))
            {
                ValidationErrors.Add($"Excel文件不存在: {_excelPath}");
                return;
            }
            try
            {
                FileInfo file = new FileInfo(_excelPath);

                using (ExcelPackage package = new ExcelPackage(file))
                {
                    if (package.Workbook.Worksheets.Count == 0)
                    {
                        ValidationErrors.Add("Excel文件中没有工作表");
                        return;
                    }
                    ExcelWorksheet worksheet = package.Workbook.Worksheets[0];

                    int rowCount = worksheet.Dimension?.Rows ?? 0;
                    int colCount = worksheet.Dimension?.Columns ?? 0;

                    if (rowCount < 2 || colCount == 0)
                    {
                        ValidationErrors.Add("Excel文件没有数据");
                        return;
                    }

                    // 读取标题行
                    _headers = new string[colCount];
                    for (int col = 1; col <= colCount; col++)
                    {
                        _headers[col - 1] = worksheet.Cells[1, col].Text?.Trim() ?? $"Column{col}";
                    }

                    // 读取数据行
                    for (int row = 2; row <= rowCount; row++)
                    {
                        var rowData = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase);
                        for (int col = 1; col <= colCount; col++)
                        {
                            string header = _headers[col - 1];
                            string cellValue = worksheet.Cells[row, col].Text?.Trim() ?? "";
                            rowData[header] = cellValue;
                        }

                        // 只添加有主键的行
                        if (rowData.ContainsKey(_primaryKeyColumn) &&
                            !string.IsNullOrWhiteSpace(rowData[_primaryKeyColumn]))
                        {
                            _rows.Add(rowData);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ValidationErrors.Add($"加载Excel数据失败: {ex.Message}");
            }
        }
        private void ValidateExcelData()
        {
            ValidationErrors.Clear();

            // 1. 检查主键配置是否存在
            if (string.IsNullOrWhiteSpace(_config.excel_primary_key))
            {
                ValidationErrors.Add("配置错误: 未指定主键列名");
            }

            // 2. 检查列映射配置
            if (_config.excel_columns == null || _config.excel_columns.Count == 0)
            {
                ValidationErrors.Add("配置错误: 未配置列映射");
            }
            else
            {
                // 检查主键列是否在映射中
                bool primaryKeyFound = false;
                foreach (var colConfig in _config.excel_columns)
                {
                    if (colConfig.Alias.Equals(_config.excel_primary_key, StringComparison.OrdinalIgnoreCase))
                    {
                        primaryKeyFound = true;
                        break;
                    }
                }

                if (!primaryKeyFound)
                {
                    ValidationErrors.Add($"配置错误: 主键列 '{_config.excel_primary_key}' 未在列映射中配置");
                }
            }

            // 3. 检查Excel列是否存在
            if (_rows.Count == 0)
            {
                ValidationErrors.Add("Excel中没有有效数据行");
                IsValid = false;
                return;
            }

            // 检查必需的原始列是否存在
            foreach (var colConfig in _config.excel_columns)
            {
                if (colConfig.IsRequired && !_headers.Contains(colConfig.OriginalName, StringComparer.OrdinalIgnoreCase))
                {
                    ValidationErrors.Add($"Excel中缺少必需的列: {colConfig.OriginalName}");
                }
            }

            // 4. 检查主键列是否存在
            bool primaryKeyColumnExists = false;
            foreach (var header in _headers)
            {
                if (header.Equals(_config.excel_primary_key, StringComparison.OrdinalIgnoreCase))
                {
                    primaryKeyColumnExists = true;
                    break;
                }
            }

            if (!primaryKeyColumnExists)
            {
                ValidationErrors.Add($"Excel中缺少主键列: {_config.excel_primary_key}");
            }

            // 5. 检查主键值是否重复
            var keyValues = new HashSet<string>(StringComparer.OrdinalIgnoreCase);
            for (int i = 0; i < _rows.Count; i++)
            {
                var row = _rows[i];
                if (row.TryGetValue(_config.excel_primary_key, out string keyValue) && !string.IsNullOrWhiteSpace(keyValue))
                {
                    if (keyValues.Contains(keyValue))
                    {
                        ValidationErrors.Add($"主键值重复: {keyValue} (行 {i + 2})");
                    }
                    else
                    {
                        keyValues.Add(keyValue);
                    }
                }
                else
                {
                    ValidationErrors.Add($"主键值为空 (行 {i + 2})");
                }
            }

            // 6. 检查必需列的值
            foreach (var colConfig in _config.excel_columns.Where(c => c.IsRequired))
            {
                for (int i = 0; i < _rows.Count; i++)
                {
                    var row = _rows[i];
                    if (!row.TryGetValue(colConfig.OriginalName, out string value) || string.IsNullOrWhiteSpace(value))
                    {
                        ValidationErrors.Add($"必需列 '{colConfig.OriginalName}' 值为空 (行 {i + 2})");
                    }
                }
            }

            IsValid = ValidationErrors.Count == 0;
        }

        // 检查主键是否已使用
        public bool IsKeyUsed(string keyValue)
        {
            return !string.IsNullOrWhiteSpace(keyValue) &&
                   _usageRecords.ContainsKey(keyValue);
        }

        // 获取使用记录
        public UsageRecord GetUsageRecord(string keyValue)
        {
            if (string.IsNullOrWhiteSpace(keyValue)) return null;
            return _usageRecords.TryGetValue(keyValue, out var record) ? record : null;
        }

        public (List<string> VariableNames, List<string> VariableValues) GetVariableListFromRow(
            Dictionary<string, string> row)
        {
            var variableNames = new List<string>();
            var variableValues = new List<string>();

            if (row == null || _config?.excel_columns == null)
                return (variableNames, variableValues);

            // 按照配置顺序处理列
            foreach (var colConfig in _config.excel_columns)
            {
                // 使用别名作为变量名
                string varName = colConfig.Alias;

                // 尝试获取值
                string value = null;
                if (row.TryGetValue(colConfig.OriginalName, out string rawValue))
                {
                    value = rawValue;
                }

                // 处理空值和默认值
                if (string.IsNullOrWhiteSpace(value))
                {
                    if (!string.IsNullOrWhiteSpace(colConfig.DefaultValue))
                    {
                        value = colConfig.DefaultValue;
                    }
                    else if (colConfig.IsRequired)
                    {
                        // 必需列但值为空，记录警告
                        OnLog?.Invoke($"必需列 '{colConfig.OriginalName}' 值为空，使用空字符串", LogLevel.error);
                        value = "";
                    }
                }

                // 添加到结果
                variableNames.Add(varName);
                variableValues.Add(value ?? "");
            }

            return (variableNames, variableValues);
        }


        // 顺序获取下一行数据
        // 顺序获取下一行数据（从未使用的位置开始）
        public Dictionary<string, string> GetNextRow(bool markAsUsed = true, string usedBy = null, string macAddress = null)
        {
            // 从未使用的位置开始查找
            int startIndex = _lastUsedIndex;
            int index = startIndex;
            int checkedCount = 0;
            if (!IsValid)
            {
                throw new InvalidOperationException("Excel数据未通过验证，无法获取数据");
            }

            while (checkedCount < _rows.Count)
            {
                // 循环遍历所有行
                if (index >= _rows.Count) index = 0;

                var row = _rows[index];
                string key = GetPrimaryKey(row);

                // 找到未使用的行
                if (!IsKeyUsed(key))
                {
                    _lastUsedIndex = index + 1; // 更新最后使用位置

                    if (markAsUsed)
                    {
                        MarkKeyAsUsed(key, usedBy, macAddress);
                    }
                    return new Dictionary<string, string>(row);
                }

                // 移动到下一行
                index++;
                checkedCount++;
            }

            // 所有行都已使用
            return null;
        }

        // 根据主键值获取整行数据
        public Dictionary<string, string> GetRowByKey(string keyValue, bool markAsUsed = false, string usedBy = null, string macAddress = null)
        {
            if (string.IsNullOrWhiteSpace(keyValue)) return null;

            var row = _rows.FirstOrDefault(r =>
                GetPrimaryKey(r).Equals(keyValue, StringComparison.OrdinalIgnoreCase));

            if (row != null)
            {
                if (markAsUsed)
                {
                    MarkKeyAsUsed(keyValue, usedBy, macAddress);
                }
                return new Dictionary<string, string>(row);
            }
            return null;
        }

        public Dictionary<string, string> FindRowByColumnValue(string columnName, string columnValue)
        {
            if (string.IsNullOrWhiteSpace(columnName)) return null;
            if (string.IsNullOrWhiteSpace(columnValue)) return null;

            var row = _rows.FirstOrDefault(r =>
                r.ContainsKey(columnName) &&
                r[columnName].Equals(columnValue, StringComparison.OrdinalIgnoreCase));

            if (row != null)
            {
                return new Dictionary<string, string>(row);
            }
            return null;
        }

        // 标记主键为已使用（带额外信息）
        public void MarkKeyAsUsed(string keyValue, string usedBy = null, string macAddress = null)
        {
            if (!string.IsNullOrWhiteSpace(keyValue))
            {
                // 如果已经标记过，更新记录
                if (_usageRecords.TryGetValue(keyValue, out var record))
                {
                    record.LastUsedTime = DateTime.Now;
                    if (!string.IsNullOrWhiteSpace(usedBy)) record.UsedBy = usedBy;
                    if (!string.IsNullOrWhiteSpace(macAddress)) record.MacAddress = macAddress;
                }
                else
                {
                    // 创建新记录
                    _usageRecords[keyValue] = new UsageRecord
                    {
                        KeyValue = keyValue,
                        FirstUsedTime = DateTime.Now,
                        LastUsedTime = DateTime.Now,
                        UsedBy = usedBy,
                        MacAddress = macAddress
                    };
                }
            }
        }

        // 根据主键值获取指定列的值
        public string GetColumnValue(string keyValue, string columnName, bool markAsUsed = false, string usedBy = null, string macAddress = null)
        {
            var row = GetRowByKey(keyValue, markAsUsed, usedBy, macAddress);
            return row != null && row.TryGetValue(columnName, out string value) ? value : null;
        }

        public string GetColumnValueByQuery(string queryColumn, string queryValue, string resultColumn, bool markAsUsed = false)
        {
            var row = FindRowByColumnValue(queryColumn, queryValue);
            return row != null && row.TryGetValue(resultColumn, out string value) ? value : null;
        }

        // 获取主键值
        private string GetPrimaryKey(Dictionary<string, string> row)
        {
            return row.TryGetValue(_primaryKeyColumn, out string value) ? value : "";
        }

        // 根据主键获取变量列表
        public (List<string> VariableNames, List<string> VariableValues) GetVariableListByKey(
            string keyValue, bool markAsUsed = false, string usedBy = null, string macAddress = null)
        {
            var row = GetRowByKey(keyValue, markAsUsed, usedBy, macAddress);
            return GetVariableListFromRow(row);
        }

        // 获取下一行的变量列表
        public (List<string> VariableNames, List<string> VariableValues) GetNextVariableList(
            bool markAsUsed = true, string usedBy = null, string macAddress = null)
        {
            var row = GetNextRow(markAsUsed, usedBy, macAddress);
            return GetVariableListFromRow(row);
        }




        // 保存进度
        public void SaveProgress()
        {
            var progress = new ProgressData
            {
                LastUsedIndex = _lastUsedIndex,
                UsageRecords = _usageRecords.Values.ToList()
            };

            File.WriteAllText(_progressPath, JsonConvert.SerializeObject(progress, Formatting.Indented));
        }

        // 加载进度
        private void LoadProgress()
        {
            if (File.Exists(_progressPath))
            {
                try
                {
                    var json = File.ReadAllText(_progressPath);
                    var progress = JsonConvert.DeserializeObject<ProgressData>(json);

                    _lastUsedIndex = progress.LastUsedIndex;

                    // 加载使用记录
                    foreach (var record in progress.UsageRecords)
                    {
                        _usageRecords[record.KeyValue] = record;
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine($"加载进度失败: {ex.Message}");
                    // 进度文件损坏时重置
                    ResetProgress();
                }
            }
        }

        // 重置进度
        public void ResetProgress()
        {
            _lastUsedIndex = 0;
            _usageRecords.Clear();
            SaveProgress();
        }

        private void LogValidationErrors()
        {
            foreach (var error in ValidationErrors)
            {
                OnLog?.Invoke(error, LogLevel.error);
            }
            OnLogShow?.Invoke("Excel数据验证失败，详情查看日志");
        }

        // 

        // 获取所有使用记录
        public List<UsageRecord> GetAllUsageRecords()
        {
            return _usageRecords.Values.ToList();
        }

        // 使用记录类
        public class UsageRecord
        {
            public string KeyValue { get; set; }
            public DateTime FirstUsedTime { get; set; }
            public DateTime LastUsedTime { get; set; }
            public string UsedBy { get; set; }
            public string MacAddress { get; set; }

            // 格式化显示
            public override string ToString()
            {
                return $"{KeyValue}: 首次使用 {FirstUsedTime:yyyy-MM-dd HH:mm}, 最后使用 {LastUsedTime:yyyy-MM-dd HH:mm}, " +
                       $"使用者: {UsedBy ?? "未知"}, MAC: {MacAddress ?? "未知"}";
            }
        }

        // 进度数据类
        private class ProgressData
        {
            public int LastUsedIndex { get; set; }
            public List<UsageRecord> UsageRecords { get; set; }
        }
    }
}
